import {Injectable} from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {Observable} from "rxjs";
import {Session, User} from "../login/utils/helper.class";
import {map} from "rxjs/operators";
import {properties} from "../../../environments/environment";
import {StringUtils} from "../utils/string-utils.class";
import {CustomOptions} from "./servicesUtils/customOptions.class";
import {AdvancedAsyncSubject} from "../utils/AdvancedAsyncSubject";

@Injectable({
  providedIn: 'root'
})
export class UserManagementService {
  private readonly getUserInfoSubject: AdvancedAsyncSubject<User> = new AdvancedAsyncSubject<User>();
  public fixRedirectURL: string = null;
  private redirectUrl: string = null;
  private subscription;
  private readonly routerSubscription;
  
  constructor(private http: HttpClient) {
    this.updateUserInfo();
  }
  
  clearSubscriptions() {
    if (this.subscription) {
      this.subscription.unsubscribe();
    }
    if (this.routerSubscription) {
      this.routerSubscription.unsubscribe();
    }
  }
  
  public get user(): User {
    return this.getUserInfoSubject.getValue();
  }
  
  public getUserInfo(): Observable<User> {
    return this.getUserInfoSubject.asObservable();
  }
  
  public updateUserInfo(resolve: Function = null) {
    this.subscription = this.http.get<User>(properties.userInfoUrl, CustomOptions.registryOptions()).pipe(map(userInfo => {
      return this.parseUserInfo(userInfo);
    })).subscribe(user => {
      this.getUserInfoSubject.next(user);
      if (resolve) {
        resolve();
      }
    }, error => {
      this.getUserInfoSubject.next(null);
      if (resolve) {
        resolve();
      }
    });
  }
  
  private parseUserInfo(info: any) {
    const user: User = new User();
    user.id = (info.sub && info.sub.indexOf('@')) ? info.sub.substring(0, info.sub.indexOf('@')) : info.sub;
    user.firstname = (info.given_name) ? info.given_name : "";
    user.lastname = (info.family_name) ? info.family_name : "";
    user.email = info.email.toLowerCase(); // TODO remove, is a quick fix
    user.fullname = (info.name) ? info.name : "";
    if (user.fullname == "") {
      if (user.firstname != "") {
        user.fullname += user.firstname;
      }
      if (user.lastname != "") {
        user.fullname += user.lastname;
      }
      if (user.fullname == "") { //fullname is still empty set a default
        user.fullname = "Anonymous user";
      }
    }
    user.role = [];
    if (info.roles) {
      info.roles.forEach(role => {
        user.role.push(role);
      });
    } else {
      if (info.edu_person_entitlements) {
        user.role = info.edu_person_entitlements;
      }
    }
    user.expirationDate = info.exp_date;
    return user;
  }
  
  public setRedirectUrl(url: string = null) {
    if (url) {
      let parts = url.split('?');
      let path = properties.baseLink + parts[0];
      let params = null;
      if (parts.length == 2) {
        params = parts[1];
      }
      let hash = path.indexOf("#");
      let fragment = (hash !== -1) ? path.slice(hash + 1) : null;
      if (fragment) {
        path = path.slice(0, hash);
      } else {
        fragment = "";
      }
      if (!path.includes('/reload')) {
        Session.setReloadUrl(location.protocol + "//" + location.host, path, params, fragment);
      }
      this.redirectUrl = StringUtils.URIEncode(location.protocol + "//" + location.host + this.fixRedirectURL);
    } else {
      this.redirectUrl = StringUtils.URIEncode(location.href);
      Session.setReloadUrl(location.protocol + "//" + location.host, location.pathname, location.search, location.hash);
    }
  }
  
  public login() {
    if (this.fixRedirectURL) {
      this.setRedirectUrl(this.fixRedirectURL);
    } else {
      this.setRedirectUrl();
    }
    window.location.href = properties.loginUrl + "?redirect=" + this.redirectUrl;
  }
  
  public logout() {
    this.setRedirectUrl();
    Session.removeUser();
    window.location.href = properties.logoutUrl + "?redirect=" + this.redirectUrl;
  }
}
